#include "mures.h"
#include "settings.h"
#include "mellinint.h"
#include "resint.h"
#include "anomalous.h"
#include "mesq.h"
#include "resconst.h"
#include "ccoeff.h"
#include "hcoeff.h"

#include <iostream>

//LL
complex <double> *mures::qqb;
//NLL
complex <double> *mures::qg;
complex <double> *mures::qg_1;
complex <double> *mures::qg_2;
//NNLL
complex <double> *mures::qq;
complex <double> *mures::qq_1;
complex <double> *mures::qq_2;
complex <double> *mures::qqp;
complex <double> *mures::qqp_1;
complex <double> *mures::qqp_2;
complex <double> *mures::qqbp;
complex <double> *mures::qqbp_1;
complex <double> *mures::qqbp_2;
complex <double> *mures::gg;
//NNNLL
complex <double> *mures::qbg;
complex <double> *mures::qbg_1;
complex <double> *mures::qbg_2;
complex <double> *mures::qpg;
complex <double> *mures::qpg_1;
complex <double> *mures::qpg_2;
complex <double> *mures::qbpg;
complex <double> *mures::qbpg_1;
complex <double> *mures::qbpg_2;
//NNNNLL
complex <double> *mures::qbq;
complex <double> *mures::qbqp;
complex <double> *mures::qbqbp;
complex <double> *mures::qpqp;
complex <double> *mures::qpqbp;
complex <double> *mures::qbpqp;

using namespace ccoeff;
using namespace resconst;
using namespace resint;


//allocate memory
void mures::allocate()
{
  if (opts.mellin1d)
    {
      qqb  = new complex <double> [mellinint::mdim*2];
      qg   = new complex <double> [mellinint::mdim*2];
      qq   = new complex <double> [mellinint::mdim*2];
      qqp  = new complex <double> [mellinint::mdim*2];
      qqbp = new complex <double> [mellinint::mdim*2];
      gg   = new complex <double> [mellinint::mdim*2];
      qbg  = new complex <double> [mellinint::mdim*2];
      qpg  = new complex <double> [mellinint::mdim*2];
      qbpg = new complex <double> [mellinint::mdim*2];
      qbq   = new complex <double> [mellinint::mdim*2];
      qbqp  = new complex <double> [mellinint::mdim*2];
      qbqbp = new complex <double> [mellinint::mdim*2];
      qpqp  = new complex <double> [mellinint::mdim*2];
      qpqbp = new complex <double> [mellinint::mdim*2];
      qbpqp = new complex <double> [mellinint::mdim*2];
    }
  else
    {
      qqb    = new complex <double> [mellinint::mdim*mellinint::mdim*2];
      qg_1   = new complex <double> [mellinint::mdim*mellinint::mdim*2];
      qg_2   = new complex <double> [mellinint::mdim*mellinint::mdim*2];
      qq_1   = new complex <double> [mellinint::mdim*mellinint::mdim*2];
      qq_2   = new complex <double> [mellinint::mdim*mellinint::mdim*2];
      qqp_1  = new complex <double> [mellinint::mdim*mellinint::mdim*2];
      qqp_2  = new complex <double> [mellinint::mdim*mellinint::mdim*2];
      qqbp_1 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
      qqbp_2 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
      gg     = new complex <double> [mellinint::mdim*mellinint::mdim*2];
      qbg_1  = new complex <double> [mellinint::mdim*mellinint::mdim*2];
      qbg_2  = new complex <double> [mellinint::mdim*mellinint::mdim*2];
      qpg_1  = new complex <double> [mellinint::mdim*mellinint::mdim*2];
      qpg_2  = new complex <double> [mellinint::mdim*mellinint::mdim*2];
      qbpg_1 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
      qbpg_2 = new complex <double> [mellinint::mdim*mellinint::mdim*2];
    }
}

void mures::free()
{
  if (opts.mellin1d)
    {
      delete[] qqb;
      delete[] qg;
      delete[] qq;
      delete[] qqp;  
      delete[] qqbp;
      delete[] gg;
      delete[] qbg;
      delete[] qpg;
      delete[] qbpg;
      delete[] qbq;
      delete[] qbqp;
      delete[] qbqbp;
      delete[] qpqp;
      delete[] qpqbp;
      delete[] qbpqp;
    }
  else
    {
      delete[] qqb;
      delete[] qg_1;
      delete[] qg_2;
      delete[] qq_1;
      delete[] qq_2;
      delete[] qqp_1;
      delete[] qqp_2;
      delete[] qqbp_1;
      delete[] qqbp_2;
      delete[] gg;
      delete[] qbg_1;
      delete[] qbg_2;
      delete[] qpg_1;
      delete[] qpg_2;
      delete[] qbpg_1;
      delete[] qbpg_2;
    }
}

void mures::reset()
{
  if (opts.mellin1d)
    {
      fill(qqb , qqb  + mellinint::mdim*2, 0.);
      fill(qg  , qg   + mellinint::mdim*2, 0.);
      fill(qq  , qq   + mellinint::mdim*2, 0.);
      fill(qqp , qqp  + mellinint::mdim*2, 0.);
      fill(qqbp, qqbp + mellinint::mdim*2, 0.);
      fill(gg  , gg   + mellinint::mdim*2, 0.);
      fill(qbg , qbg  + mellinint::mdim*2, 0.);
      fill(qpg , qpg  + mellinint::mdim*2, 0.);
      fill(qbpg, qbpg + mellinint::mdim*2, 0.);
      fill(qbq  ,qbq   + mellinint::mdim*2, 0.);
      fill(qbqp ,qbqp  + mellinint::mdim*2, 0.);
      fill(qbqbp,qbqbp + mellinint::mdim*2, 0.);
      fill(qpqp ,qpqp  + mellinint::mdim*2, 0.);
      fill(qpqbp,qpqbp + mellinint::mdim*2, 0.);
      fill(qbpqp,qbpqp + mellinint::mdim*2, 0.);
    }
  else
    {
      fill(qqb   ,qqb    + mellinint::mdim*mellinint::mdim*2, 0.);
      fill(qg_1  ,qg_1   + mellinint::mdim*mellinint::mdim*2, 0.);
      fill(qg_2  ,qg_2   + mellinint::mdim*mellinint::mdim*2, 0.);
      fill(qq_1  ,qq_1   + mellinint::mdim*mellinint::mdim*2, 0.);
      fill(qq_2  ,qq_2   + mellinint::mdim*mellinint::mdim*2, 0.);
      fill(qqp_1 ,qqp_1  + mellinint::mdim*mellinint::mdim*2, 0.);
      fill(qqp_2 ,qqp_2  + mellinint::mdim*mellinint::mdim*2, 0.);
      fill(qqbp_1,qqbp_1 + mellinint::mdim*mellinint::mdim*2, 0.);
      fill(qqbp_2,qqbp_2 + mellinint::mdim*mellinint::mdim*2, 0.);
      fill(gg    ,gg     + mellinint::mdim*mellinint::mdim*2, 0.);
      fill(qbg_1 ,qbg_1  + mellinint::mdim*mellinint::mdim*2, 0.);
      fill(qbg_2 ,qbg_2  + mellinint::mdim*mellinint::mdim*2, 0.);
      fill(qpg_1 ,qpg_1  + mellinint::mdim*mellinint::mdim*2, 0.);
      fill(qpg_2 ,qpg_2  + mellinint::mdim*mellinint::mdim*2, 0.);
      fill(qbpg_1,qbpg_1 + mellinint::mdim*mellinint::mdim*2, 0.);
      fill(qbpg_2,qbpg_2 + mellinint::mdim*mellinint::mdim*2, 0.);
    }
}

void mures::calc()
{
  //Resummation scale variations should match the order of the Sudakov (i.e. they should match the resummation order at unprimed orders)
  if (opts.order_sudak == 0)
    return;

  double LQ2 = pow(LQ,2);
  double LQ3 = pow(LQ,3);
  double LQ4 = pow(LQ,4);
  double LQ5 = pow(LQ,5);
  double LQ6 = pow(LQ,6);
  double LQ7 = pow(LQ,7);
  double LQ8 = pow(LQ,8);
  double LQ9 = pow(LQ,9);
  double LQ10= pow(LQ,10);

  double LR2 = pow(LR,2);

  double as = aass;
  double as2 = pow(as,2);
  double as3 = pow(as,3);
  double as4 = pow(as,4);
  double as5 = pow(as,5);

  double A1 = A1q;
  double A2 = A2q;
  double A3 = A3q;
  double A4 = A4q;
  double A5 = A5q;

  double B1 = B1q;
  double B2 = B2q;
  double B3 = B3q;
  double B4 = B4q;
  double B5 = B5q;

  //h5q_1, h5q_2, h5q_3 formulas are guessed
  double h1q_0 =            - B1*LQ - 1./2.*A1*LQ2;
  double h2q_0 =   beta0*LQ - B1*LQ - 1./2.*A1*LQ2;
  double h3q_0 = 2*beta0*LQ - B1*LQ - 1./2.*A1*LQ2;
  double h4q_0 = 3*beta0*LQ - B1*LQ - 1./2.*A1*LQ2;
  double h5q_0 = 4*beta0*LQ - B1*LQ - 1./2.*A1*LQ2;
  
  double h2q_1 =                                 - B2*LQ - 1./2.*B1*beta0*LQ2 + 1./2.*pow(B1,2)*LQ2 - 1./2.*A2*LQ2 - 1./3.*A1*beta0*LQ3 + 1./2.*A1*B1*LQ3 + 1./8.*pow(A1,2)*LQ4;
  double h3q_1 =   beta1*LQ +   pow(beta0,2)*LQ2 - B2*LQ - 3./2.*B1*beta0*LQ2 + 1./2.*pow(B1,2)*LQ2 - 1./2.*A2*LQ2 - 5./6.*A1*beta0*LQ3 + 1./2.*A1*B1*LQ3 + 1./8.*pow(A1,2)*LQ4;
  double h4q_1 = 2*beta1*LQ + 3*pow(beta0,2)*LQ2 - B2*LQ - 5./2.*B1*beta0*LQ2 + 1./2.*pow(B1,2)*LQ2 - 1./2.*A2*LQ2 - 4./3.*A1*beta0*LQ3 + 1./2.*A1*B1*LQ3 + 1./8.*pow(A1,2)*LQ4;
  double h5q_1 = 3*beta1*LQ + 5*pow(beta0,2)*LQ2 - B2*LQ - 7./2.*B1*beta0*LQ2 + 1./2.*pow(B1,2)*LQ2 - 1./2.*A2*LQ2 -11./6.*A1*beta0*LQ3 + 1./2.*A1*B1*LQ3 + 1./8.*pow(A1,2)*LQ4;

  double h3q_2 =                                                           - B3*LQ -   B2*beta0*LQ2 - 1./2.*B1*beta1*LQ2 -  1./3.*B1*pow(beta0,2)*LQ3 + B1*B2*LQ2 + 1./2.*pow(B1,2)*beta0*LQ3 - 1./6.*pow(B1,3)*LQ3 - 1./2.*A3*LQ2 - 2./3.*A2*beta0*LQ3 + 1./2.*A2*B1*LQ3 - 1./3.*A1*beta1*LQ3 -   1./4.*A1*pow(beta0,2)*LQ4 + 1./2.*A1*B2*LQ3 +  7./12.*A1*B1*beta0*LQ4 - 1./4.*A1*pow(B1,2)*LQ4 + 1./4.*A1*A2*LQ4 +  1./6.*pow(A1,2)*beta0*LQ5 - 1./8.*pow(A1,2)*B1*LQ5 - 1./48.*pow(A1,3)*LQ6;
  double h4q_2 =   beta2*LQ +  5./2.*beta0*beta1*LQ2 +    pow(beta0,3)*LQ3 - B3*LQ - 2*B2*beta0*LQ2 - 3./2.*B1*beta1*LQ2 - 11./6.*B1*pow(beta0,2)*LQ3 + B1*B2*LQ2 +       pow(B1,2)*beta0*LQ3 - 1./6.*pow(B1,3)*LQ3 - 1./2.*A3*LQ2 - 7./6.*A2*beta0*LQ3 + 1./2.*A2*B1*LQ3 - 5./6.*A1*beta1*LQ3 - 13./12.*A1*pow(beta0,2)*LQ4 + 1./2.*A1*B2*LQ3 + 13./12.*A1*B1*beta0*LQ4 - 1./4.*A1*pow(B1,2)*LQ4 + 1./4.*A1*A2*LQ4 + 7./24.*pow(A1,2)*beta0*LQ5 - 1./8.*pow(A1,2)*B1*LQ5 - 1./48.*pow(A1,3)*LQ6;
  double h5q_2 = 2*beta2*LQ + 15./2.*beta0*beta1*LQ2 + 3.*pow(beta0,3)*LQ3 - B3*LQ - 3*B2*beta0*LQ2 - 5./2.*B1*beta1*LQ2 - 20./6.*B1*pow(beta0,2)*LQ3 + B1*B2*LQ2 + 3./2.*pow(B1,2)*beta0*LQ3 - 1./6.*pow(B1,3)*LQ3 - 1./2.*A3*LQ2 -10./6.*A2*beta0*LQ3 + 1./2.*A2*B1*LQ3 - 8./6.*A1*beta1*LQ3 - 23./12.*A1*pow(beta0,2)*LQ4 + 1./2.*A1*B2*LQ3 + 18./12.*A1*B1*beta0*LQ4 - 1./4.*A1*pow(B1,2)*LQ4 + 1./4.*A1*A2*LQ4 +10./24.*pow(A1,2)*beta0*LQ5 - 1./8.*pow(A1,2)*B1*LQ5 - 1./48.*pow(A1,3)*LQ6;

  double h4q_3 =                                                                                                           - B4*LQ - 3./2.*B3*beta0*LQ2 -   B2*beta1*LQ2 - B2*pow(beta0,2)*LQ3 + 1./2.*pow(B2,2)*LQ2 - 1./2.*B1*beta2*LQ2 - 5./6.*B1*beta0*beta1*LQ3 - 1./4.*B1*pow(beta0,3)*LQ4 + B1*B3*LQ2 + 3./2.*B1*B2*beta0*LQ3 + 1./2.*pow(B1,2)*beta1*LQ3 + 11./24.*pow(B1,2)*pow(beta0,2)*LQ4 - 1./2.*pow(B1,2)*B2*LQ3 - 1./4.*pow(B1,3)*beta0*LQ4 + 1./24.*pow(B1,4)*LQ4 - 1./2.*A4*LQ2 - A3*beta0*LQ3 + 1./2.*A3*B1*LQ3 - 2./3.*A2*beta1*LQ3 - 3./4.*A2*pow(beta0,2)*LQ4 + 1./2.*A2*B2*LQ3 + 11./12.*A2*B1*beta0*LQ4 - 1./4.*A2*pow(B1,2)*LQ4 + 1./8.*pow(A2,2)*LQ4 - 1./3.*A1*beta2*LQ3 - 5./8.*A1*beta0*beta1*LQ4 - 1./5.*A1*pow(beta0,3)*LQ5 + 1./2.*A1*B3*LQ3 + 5./6.*A1*B2*beta0*LQ4 + 7./12.*A1*B1*beta1*LQ4 + 7./12.*A1*B1*pow(beta0,2)*LQ5 - 1./2.*A1*B1*B2*LQ4 - 5./12.*A1*pow(B1,2)*beta0*LQ5 + 1./12.*A1*pow(B1,3)*LQ5 + 1./4.*A1*A3*LQ4 + 1./2.*A1*A2*beta0*LQ5 - 1./4.*A1*A2*B1*LQ5 + 1./6.*pow(A1,2)*beta1*LQ5 + 13./72.*pow(A1,2)*pow(beta0,2)*LQ6 - 1./8.*pow(A1,2)*B2*LQ5 - 11./48.*pow(A1,2)*B1*beta0*LQ6 + 1./16.*pow(A1,2)*pow(B1,2)*LQ6 - 1./16.*pow(A1,2)*A2*LQ6 - 1./24.*pow(A1,3)*beta0*LQ7 + 1./48.*pow(A1,3)*B1*LQ7 + 1./384.*pow(A1,4)*LQ8;
  double h5q_3 = beta3*LQ + 3./2.*pow(beta1,2)*LQ2 + 3.*beta0*beta2*LQ2 + 13./3.*pow(beta0,2)*beta1*LQ3 + pow(beta0,4)*LQ4 - B4*LQ - 5./2.*B3*beta0*LQ2 - 2*B2*beta1*LQ2 - B2*pow(beta0,2)*LQ3 + 1./2.*pow(B2,2)*LQ2 - 3./2.*B1*beta2*LQ2 - 5./6.*B1*beta0*beta1*LQ3 - 1./4.*B1*pow(beta0,3)*LQ4 + B1*B3*LQ2 + 3./2.*B1*B2*beta0*LQ3 + 1./2.*pow(B1,2)*beta1*LQ3 + 11./24.*pow(B1,2)*pow(beta0,2)*LQ4 - 1./2.*pow(B1,2)*B2*LQ3 - 1./4.*pow(B1,3)*beta0*LQ4 + 1./24.*pow(B1,4)*LQ4 - 1./2.*A4*LQ2 - A3*beta0*LQ3 + 1./2.*A3*B1*LQ3 - 2./3.*A2*beta1*LQ3 - 3./4.*A2*pow(beta0,2)*LQ4 + 1./2.*A2*B2*LQ3 + 11./12.*A2*B1*beta0*LQ4 - 1./4.*A2*pow(B1,2)*LQ4 + 1./8.*pow(A2,2)*LQ4 - 5./6.*A1*beta2*LQ3 - 5./8.*A1*beta0*beta1*LQ4 - 1./5.*A1*pow(beta0,3)*LQ5 + 1./2.*A1*B3*LQ3 + 5./6.*A1*B2*beta0*LQ4 + 7./12.*A1*B1*beta1*LQ4 + 7./12.*A1*B1*pow(beta0,2)*LQ5 - 1./2.*A1*B1*B2*LQ4 - 5./12.*A1*pow(B1,2)*beta0*LQ5 + 1./12.*A1*pow(B1,3)*LQ5 + 1./4.*A1*A3*LQ4 + 1./2.*A1*A2*beta0*LQ5 - 1./4.*A1*A2*B1*LQ5 + 1./6.*pow(A1,2)*beta1*LQ5 + 13./72.*pow(A1,2)*pow(beta0,2)*LQ6 - 1./8.*pow(A1,2)*B2*LQ5 - 11./48.*pow(A1,2)*B1*beta0*LQ6 + 1./16.*pow(A1,2)*pow(B1,2)*LQ6 - 1./16.*pow(A1,2)*A2*LQ6 - 1./24.*pow(A1,3)*beta0*LQ7 + 1./48.*pow(A1,3)*B1*LQ7 + 1./384.*pow(A1,4)*LQ8;

  double h5q_4 = - B5*LQ - 2*B4*beta0*LQ2 - 3./2.*B3*beta1*LQ2 - 2*B3*pow(beta0,2)*LQ3 + B2*beta2*LQ2 - 7./3.*B2*beta0*beta1*LQ3 - B2*pow(beta0,3)*LQ4 + B2*B3*LQ2 + pow(B2,2)*beta0*LQ3 - 1./2.*B1*beta3*LQ2 - 1./2.*B1*pow(beta1,2)*LQ3 - B1*beta0*beta2*LQ3 - 13./12.*B1*pow(beta0,2)*beta1*LQ4 - 1./5.*B1*pow(beta0,4)*LQ5 + B1*B4*LQ2 + 2*B1*B3*beta0*LQ3 + 3./2.*B1*B2*beta1*LQ3 + 11./6.*B1*B2*pow(beta0,2)*LQ4 - 1./2.*B1*pow(B2,2)*LQ3 + 1./2.*pow(B1,2)*beta2*LQ3 + 13./12.*pow(B1,2)*beta0*beta1*LQ4 + 5./12.*pow(B1,2)*pow(beta0,3)*LQ5 - 1./2.*pow(B1,2)*B3*LQ3 - pow(B1,2)*B2*beta0*LQ4 - 1./4.*pow(B1,3)*beta1*LQ4 - 7./24.*pow(B1,3)*pow(beta0,2)*LQ5 + 1./6.*pow(B1,3)*B2*LQ4 + 1./12.*pow(B1,4)*beta0*LQ5 - 1./120.*pow(B1,5)*LQ5 - 1./2.*A5*LQ2 - 4./3.*A4*beta0*LQ3 + 1./2.*A4*B1*LQ3 - A3*beta1*LQ3 - 3./2.*A3*pow(beta0,2)*LQ4 + 1./2.*A3*B2*LQ3 + 5./4.*A3*B1*beta0*LQ4 - 1./4.*A3*pow(B1,2)*LQ4 - 7./4.*A2*beta0*beta1*LQ4 - 4./5.*A2*pow(beta0,3)*LQ5 + 1./2.*A2*B3*LQ3 + 7./6.*A2*B2*beta0*LQ4 + 11./12.*A2*B1*beta1*LQ4 + 5./4.*A2*B1*pow(beta0,2)*LQ5 - 1./2.*A2*B1*B2*LQ4 - 7./12.*A2*pow(B1,2)*beta0*LQ5 + 1./12.*A2*pow(B1,3)*LQ5 + 1./4.*A2*A3*LQ4 + 1./3.*pow(A2,2)*beta0*LQ5 - 1./8.*pow(A2,2)*B1*LQ5 - 1./3.*A1*beta3*LQ3 - 3./8.*A1*pow(beta1,2)*LQ4 - 3./4.*A1*beta0*beta2*LQ4 - 13./15.*A1*pow(beta0,2)*beta1*LQ5 + 1./2.*A1*B4*LQ3 + 13./12.*A1*B3*beta0*LQ4 + 5./6.*A1*B2*beta1*LQ4 + 13./12.*A1*B2*pow(beta0,2)*LQ5 - 1./4.*A1*pow(B2,2)*LQ4 + 7./12.*A1*B1*beta2*LQ4 + 11./8.*A1*B1*beta0*beta1*LQ5 + 101./180.*A1*B1*pow(beta0,3)*LQ6 - 1./2.*A1*B1*B3*LQ4 - 13./12.*A1*B1*B2*beta0*LQ5 - 5./12.*A1*pow(B1,2)*beta1*LQ5 - 25./48.*A1*pow(B1,2)*pow(beta0,2)*LQ6 + 1./4.*A1*pow(B1,2)*B2*LQ5 + 13./72.*A1*pow(B1,3)*beta0*LQ6 - 1./48.*A1*pow(B1,4)*LQ6 + 1./4.*A1*A4*LQ4 + 2./3.*A1*A3*beta0*LQ5 - 1./4.*A1*A3*B1*LQ5 + 1./2.*A1*A2*beta1*LQ5 + 13./18.*A1*A2*pow(beta0,2)*LQ6 - 1./4.*A1*A2*B2*LQ5 - 5./8.*A1*A2*B1*beta0*LQ6 + 1./8.*A1*A2*pow(B1,2)*LQ6 - 1./16.*A1*pow(A2,2)*LQ6 + 1./6.*pow(A1,2)*beta2*LQ5 + 61./144.*pow(A1,2)*beta0*beta1*LQ6 + 11./60.*pow(A1,2)*pow(beta0,3)*LQ7 - 1./8.*pow(A1,2)*B3*LQ5 - 7./24.*pow(A1,2)*B2*beta0*LQ6 - 11./48.*pow(A1,2)*B1*beta1*LQ6 - 11./36.*pow(A1,2)*B1*pow(beta0,2)*LQ7 + 1./8.*pow(A1,2)*B1*B2*LQ6 + 7./48.*pow(A1,2)*pow(B1,2)*beta0*LQ7 - 1./48.*pow(A1,2)*pow(B1,3)*LQ7 - 1./16.*pow(A1,2)*A3*LQ6 - 1./6.*pow(A1,2)*A2*beta0*LQ7 + 1./16.*pow(A1,2)*A2*B1*LQ7 - 1./24.*pow(A1,3)*beta1*LQ7 - 17./288.*pow(A1,3)*pow(beta0,2)*LQ8 + 1./48.*pow(A1,3)*B2*LQ7 + 5./96.*pow(A1,3)*B1*beta0*LQ8 - 1./96.*pow(A1,3)*pow(B1,2)*LQ8 + 1./96.*pow(A1,3)*A2*LQ8 + 1./144.*pow(A1,4)*beta0*LQ9 - 1./384.*pow(A1,4)*B1*LQ9 - 1./3840.*pow(A1,5)*LQ10;
  
  if (opts.mellin1d)
    //rapidity integrated
    for (int i = 0; i < mellinint::mdim; i++)
      {
	int idx = anomalous::index(i,mesq::positive);

	//NLO
	complex <double> H0st_mu0_qqb = 1;
	complex <double> H1st_qqb = h1q_0 * H0st_mu0_qqb;
	
	qqb[i] += as*H1st_qqb;

	if (opts.order_sudak == 1)
	  continue;

	//NNLO
	complex <double> H1st_mu0_qqb = 2.*C1qq[idx] + H1q;
	complex <double> H1st_mu0_qg  = C1qg[idx];
	
	complex <double> H2st_qqb = h2q_1 * H0st_mu0_qqb + h2q_0 * H1st_mu0_qqb - beta0*LQ*H1q;
	complex <double> H2st_qg  =                        h2q_0 * H1st_mu0_qg ;
	
	//Mixed resummation-renormalization scale variations
	if (opts.order_hcoef >= 2)
	  H2st_qqb += -beta0*H1st_qqb*LR;

	qqb[i]  += as2*H2st_qqb;
	qg[i]   += as2*H2st_qg;

	if (opts.order_sudak == 2)
	  continue;

	//NNNLO
	complex <double> H2st_mu0_qqb  = 2.*C2qq[idx] + C1qq[idx]*C1qq[idx];
	complex <double> H2st_mu0_qg   = C2qg[idx] + C1qq[idx]*C1qg[idx];
	complex <double> H2st_mu0_qq   = C2qqb[idx];
	complex <double> H2st_mu0_qqp  = C2qqbp[idx];
	complex <double> H2st_mu0_qqbp = C2qqp[idx];
	complex <double> H2st_mu0_gg   = C1qg[idx]*C1qg[idx];

	complex <double> H3st_qqb  = h3q_2 * H0st_mu0_qqb + h3q_1 * H1st_mu0_qqb + h3q_0 * H2st_mu0_qqb ;
	complex <double> H3st_qg   =                        h3q_1 * H1st_mu0_qg  + h3q_0 * H2st_mu0_qg  ;
      	complex <double> H3st_qq   =                                               h3q_0 * H2st_mu0_qq  ;
	complex <double> H3st_qqp  =                                               h3q_0 * H2st_mu0_qqp ;
	complex <double> H3st_qqbp =                                               h3q_0 * H2st_mu0_qqbp;
	complex <double> H3st_gg   =                                               h3q_0 * H2st_mu0_gg  ;

	//Mixed resummation-renormalization scale variations
	if (opts.order_hcoef >= 3)
	  {
	    H3st_qqb  += -2*beta0*H2st_qqb*LR - (beta1*LR + pow(beta0,2)*pow(LR,2))*H1st_qqb;
	    H3st_qg   += -2*beta0*H2st_qg*LR;
	  }

	qqb[i]  += as3*H3st_qqb;
	qg[i]   += as3*H3st_qg;
	qq[i]   += as3*H3st_qq;
	qqp[i]  += as3*H3st_qqp;
	qqbp[i] += as3*H3st_qqbp;
	gg[i]   += as3*H3st_gg;


	if (opts.order_sudak == 3)
	  continue;

	//NNNNLO
	complex <double> H3st_mu0_qqb  = 2.*C3qq[idx] + 2.*C1qq[idx]*C2qq[idx];
	complex <double> H3st_mu0_qg   = C3qg[idx] + C1qq[idx]*C2qg[idx] + C2qq[idx]*C1qg[idx];
	complex <double> H3st_mu0_qq   = C3qqb[idx] + C1qq[idx]*C2qqb[idx];
	complex <double> H3st_mu0_qqp  = C3qqbp[idx] + C1qq[idx]*C2qqbp[idx];
	complex <double> H3st_mu0_qqbp = C3qqp[idx] + C1qq[idx]*C2qqp[idx];
	complex <double> H3st_mu0_gg   = 2.*C1qg[idx]*C2qg[idx];
	complex <double> H3st_mu0_qbg  = C1qg[idx]*C2qqb[idx];
	complex <double> H3st_mu0_qpg  = C1qg[idx]*C2qqp[idx];
	complex <double> H3st_mu0_qbpg = C1qg[idx]*C2qqbp[idx];

	complex <double> H4st_qqb  = h4q_3 * H0st_mu0_qqb + h4q_2 * H1st_mu0_qqb + h4q_1 * H2st_mu0_qqb + h4q_0 * H3st_mu0_qqb ;
	complex <double> H4st_qg   =                        h4q_2 * H1st_mu0_qg  + h4q_1 * H2st_mu0_qg  + h4q_0 * H3st_mu0_qg  ;
	complex <double> H4st_qq   =                                               h4q_1 * H2st_mu0_qq  + h4q_0 * H3st_mu0_qq  ;
	complex <double> H4st_qqp  =                                               h4q_1 * H2st_mu0_qqp + h4q_0 * H3st_mu0_qqp ;
	complex <double> H4st_qqbp =                                               h4q_1 * H2st_mu0_qqbp+ h4q_0 * H3st_mu0_qqbp;
	complex <double> H4st_gg   =                                               h4q_1 * H2st_mu0_gg  + h4q_0 * H3st_mu0_gg  ;
	complex <double> H4st_qbg  =                                                                      h4q_0 * H3st_mu0_qbg ;
	complex <double> H4st_qpg  =                                                                      h4q_0 * H3st_mu0_qpg ;
	complex <double> H4st_qbpg =                                                                      h4q_0 * H3st_mu0_qbpg;

	//Additional Cgq and Cgg terms
	//H4st_qqb  += -       beta0*C2qg[idx]*C1gq[idx]*LQ +       beta0*C1qg[idx]*C2gq[idx]*LQ;
	//H4st_qg   += - 1./2.*beta0*C2qg[idx]*C1gg[idx]*LQ + 1./2.*beta0*C1qg[idx]*C2gg[idx]*LQ;
	//H4st_qq   += - 1./2.*beta0*C2qg[idx]*C1gq[idx]*LQ + 1./2.*beta0*C1qg[idx]*C2gq[idx]*LQ;
	//H4st_qqp  += - 1./2.*beta0*C2qg[idx]*C1gq[idx]*LQ + 1./2.*beta0*C1qg[idx]*C2gq[idx]*LQ;
	//H4st_qqbp += - 1./2.*beta0*C2qg[idx]*C1gq[idx]*LQ + 1./2.*beta0*C1qg[idx]*C2gq[idx]*LQ;

	//Mixed resummation-renormalization scale variations
	if (opts.order_hcoef >= 4)
	  {
	    H4st_qqb  += -3*beta0*H3st_qqb*LR  -(2*beta1*LR + 3*pow(beta0,2)*pow(LR,2))*H2st_qqb - (beta2*LR + 5./2.*beta1*beta0*pow(LR,2) + pow(beta0,3)*pow(LR,3))*H1st_qqb;
	    H4st_qg   += -3*beta0*H3st_qg*LR   -(2*beta1*LR + 3*pow(beta0,2)*pow(LR,2))*H2st_qg;
	    H4st_qq   += -3*beta0*H3st_qq*LR;
	    H4st_qqp  += -3*beta0*H3st_qqp*LR;
	    H4st_qqbp += -3*beta0*H3st_qqbp*LR;
	    H4st_gg   += -3*beta0*H3st_gg*LR;
	  }
	
	qqb[i]  += as4*H4st_qqb;
	qg[i]   += as4*H4st_qg;
	qq[i]   += as4*H4st_qq;
	qqp[i]  += as4*H4st_qqp;
	qqbp[i] += as4*H4st_qqbp;
	gg[i]   += as4*H4st_gg;
	qbg[i]  += as4*H4st_qbg;
	qpg[i]  += as4*H4st_qpg;
	qbpg[i] += as4*H4st_qbpg;

	if (opts.order_sudak == 4)
	  continue;

	//NNNNNLO
	complex <double> H4st_mu0_qqb  = 2.*C4qq[idx] + 2.*(C1qq[idx]*C3qq[idx]) + C2qq[idx]*C2qq[idx];
	complex <double> H4st_mu0_qg   = C4qg[idx] + C1qq[idx]*C3qg[idx] + C3qq[idx]*C1qg[idx] + C2qq[idx]*C2qg[idx];
	complex <double> H4st_mu0_qq   = C4qqb[idx]  + C1qq[idx]*C3qqb[idx]  + C2qq[idx]*C2qqb[idx];
	complex <double> H4st_mu0_qqp  = C4qqbp[idx] + C1qq[idx]*C3qqbp[idx] + C2qq[idx]*C2qqbp[idx];
	complex <double> H4st_mu0_qqbp = C4qqp[idx]  + C1qq[idx]*C3qqp[idx]  + C2qq[idx]*C2qqp[idx];
	complex <double> H4st_mu0_gg   = 2.*(C1qg[idx]*C3qg[idx]) + C2qg[idx]*C2qg[idx];
	complex <double> H4st_mu0_qbg  = C3qqb[idx]*C1qg[idx] + C2qqb[idx]*C2qg[idx];
	complex <double> H4st_mu0_qpg  = C3qqp[idx]*C1qg[idx] + C2qqp[idx]*C2qg[idx];
	complex <double> H4st_mu0_qbpg = C3qqbp[idx]*C1qg[idx]+ C2qqbp[idx]*C2qg[idx];
	complex <double> H4st_mu0_qbq  = C2qqb[idx]*C2qqb[idx];
	complex <double> H4st_mu0_qpqbp= C2qqp[idx]*C2qqp[idx];
	complex <double> H4st_mu0_qbpqp= C2qqbp[idx]*C2qqbp[idx];
	complex <double> H4st_mu0_qbqp = C2qqb[idx]*C2qqbp[idx];
	complex <double> H4st_mu0_qbqbp= C2qqb[idx]*C2qqp[idx];
	complex <double> H4st_mu0_qpqp = C2qqp[idx]*C2qqbp[idx];

	complex <double> H5st_qqb  =  h5q_4 * H0st_mu0_qqb + h5q_3 * H1st_mu0_qqb + h5q_2 * H2st_mu0_qqb + h5q_1 * H3st_mu0_qqb  + h5q_0 * H4st_mu0_qqb  ;
	complex <double> H5st_qg   =                         h5q_3 * H1st_mu0_qg  + h5q_2 * H2st_mu0_qg  + h5q_1 * H3st_mu0_qg   + h5q_0 * H4st_mu0_qg   ;
	complex <double> H5st_qq   =                                                h5q_2 * H2st_mu0_qq  + h5q_1 * H3st_mu0_qq   + h5q_0 * H4st_mu0_qq   ;
	complex <double> H5st_qqp  =                                                h5q_2 * H2st_mu0_qqp + h5q_1 * H3st_mu0_qqp  + h5q_0 * H4st_mu0_qqp  ;
	complex <double> H5st_qqbp =                                                h5q_2 * H2st_mu0_qqbp+ h5q_1 * H3st_mu0_qqbp + h5q_0 * H4st_mu0_qqbp ;
	complex <double> H5st_gg   =                                                h5q_2 * H2st_mu0_gg  + h5q_1 * H3st_mu0_gg   + h5q_0 * H4st_mu0_gg   ;
	complex <double> H5st_qbg  =                                                                       h5q_1 * H3st_mu0_qbg  + h5q_0 * H4st_mu0_qbg  ;
	complex <double> H5st_qpg  =                                                                       h5q_1 * H3st_mu0_qpg  + h5q_0 * H4st_mu0_qpg  ;
	complex <double> H5st_qbpg =                                                                       h5q_1 * H3st_mu0_qbpg + h5q_0 * H4st_mu0_qbpg ;
	complex <double> H5st_qbq  = 												 + h5q_0 * H4st_mu0_qbq  ;
	complex <double> H5st_qpqbp= 												 + h5q_0 * H4st_mu0_qpqbp;
	complex <double> H5st_qbpqp= 												 + h5q_0 * H4st_mu0_qbpqp;
	complex <double> H5st_qbqp = 												 + h5q_0 * H4st_mu0_qbqp ;
	complex <double> H5st_qbqbp= 												 + h5q_0 * H4st_mu0_qbqbp;
	complex <double> H5st_qpqp =                                                                                             + h5q_0 * H4st_mu0_qpqp ;

	qqb[i]  += as5*H5st_qqb;
	qg[i]   += as5*H5st_qg;
	qq[i]   += as5*H5st_qq;
	qqp[i]  += as5*H5st_qqp;
	qqbp[i] += as5*H5st_qqbp;
	gg[i]   += as5*H5st_gg;
	qbg[i]  += as5*H5st_qbg;
	qpg[i]  += as5*H5st_qpg;
	qbpg[i] += as5*H5st_qbpg;
	qbq[i]  += as5*H5st_qbq  ;
	qpqbp[i]+= as5*H5st_qpqbp;
	qbpqp[i]+= as5*H5st_qbpqp;
	qbqp[i] += as5*H5st_qbqp ;
	qbqbp[i]+= as5*H5st_qbqbp;
	qpqp[i] += as5*H5st_qpqp ;
      }
  else
    //rapidity dependent
    for (int sign = mesq::positive; sign <= mesq::negative; sign++)
      for (int i1 = 0; i1 < mellinint::mdim; i1++)
	for (int i2 = 0; i2 < mellinint::mdim; i2++)
	  {
	    int ii1 = anomalous::index(i1,mesq::positive);
	    int ii2 = anomalous::index(i2,sign);
	    int idx = hcoeff::index(i1,i2,sign);

	    //NLO
	    complex <double> H0st_mu0_qqb = 1;
	    complex <double> H1st_qqb = h1q_0 * H0st_mu0_qqb;

	    qqb[idx]   += as*H1st_qqb;
	    
	    if (opts.order_sudak == 1)
	      continue;
	    
	    //NNLO
	    complex <double> H1st_mu0_qqb   = C1qq_1[ii1] + C1qq_2[ii2];
	    complex <double> H1st_mu0_qg_1  = C1qg_1[ii1];
	    complex <double> H1st_mu0_qg_2  = C1qg_2[ii2];

	    complex <double> H2st_qqb   = h2q_1 * H0st_mu0_qqb + h2q_0 * H1st_mu0_qqb ;
	    complex <double> H2st_qg_1  =                        h2q_0 * H1st_mu0_qg_1;
	    complex <double> H2st_qg_2  =                        h2q_0 * H1st_mu0_qg_2;

	    //Mixed resummation-renormalization scale variations
	    if (opts.order_hcoef >= 2)
	      H2st_qqb  += - beta0*H1st_qqb*LR;

	    qqb[idx]    += as2*H2st_qqb;
	    qg_1[idx]   += as2*H2st_qg_1;
	    qg_2[idx]   += as2*H2st_qg_2;
	    
	    if (opts.order_sudak == 2)
	      continue;

	    //NNNLO
	    complex <double> H2st_mu0_qqb    = C2qq_1[ii1] + C2qq_2[ii2] + C1qq_1[ii1]*C1qq_2[ii2];
	    complex <double> H2st_mu0_qg_1   = C2qg_1[ii1] + C1qq_2[ii2]*C1qg_1[ii1];
	    complex <double> H2st_mu0_qg_2   = C2qg_2[ii2] + C1qq_1[ii1]*C1qg_2[ii2];
	    complex <double> H2st_mu0_qq_1   = C2qqb_1[ii1];
	    complex <double> H2st_mu0_qq_2   = C2qqb_2[ii2];
	    complex <double> H2st_mu0_qqbp_1 = C2qqp_1[ii1];
	    complex <double> H2st_mu0_qqbp_2 = C2qqp_2[ii2];
	    complex <double> H2st_mu0_qqp_1  = C2qqbp_1[ii1];
	    complex <double> H2st_mu0_qqp_2  = C2qqbp_2[ii2];
	    complex <double> H2st_mu0_gg     = C1qg_1[ii1]*C1qg_2[ii2];

	    complex <double> H3st_qqb    = h3q_2 * H0st_mu0_qqb + h3q_1 * H1st_mu0_qqb  + h3q_0 * H2st_mu0_qqb   ;
	    complex <double> H3st_qg_1   =                        h3q_1 * H1st_mu0_qg_1 + h3q_0 * H2st_mu0_qg_1  ;
	    complex <double> H3st_qg_2   =                        h3q_2 * H1st_mu0_qg_2 + h3q_0 * H2st_mu0_qg_2  ;
	    complex <double> H3st_qq_1   =                                                h3q_0 * H2st_mu0_qq_1  ;
	    complex <double> H3st_qq_2   =                                                h3q_0 * H2st_mu0_qq_2  ;
	    complex <double> H3st_qqp_1  =                                                h3q_0 * H2st_mu0_qqp_1 ;
	    complex <double> H3st_qqp_2  =                                                h3q_0 * H2st_mu0_qqp_2 ;
	    complex <double> H3st_qqbp_1 =                                                h3q_0 * H2st_mu0_qqbp_1;
	    complex <double> H3st_qqbp_2 =                                                h3q_0 * H2st_mu0_qqbp_2;
	    complex <double> H3st_gg     =                                                h3q_0 * H2st_mu0_gg    ;
	    
	    //Mixed resummation-renormalization scale variations
	    if (opts.order_hcoef >= 3)
	      {
		H3st_qqb    += -2*beta0*H2st_qqb*LR - (beta1*LR + pow(beta0,2)*LR2)*H1st_qqb;
		H3st_qg_1   += -2*beta0*H2st_qg_1*LR;
		H3st_qg_2   += -2*beta0*H2st_qg_2*LR;
	      }
	    
	    qqb[idx]    += as3*H3st_qqb;
	    qg_1[idx]   += as3*H3st_qg_1;
	    qg_2[idx]   += as3*H3st_qg_2;
	    qq_1[idx]   += as3*H3st_qq_1;
	    qq_2[idx]   += as3*H3st_qq_2;
	    qqp_1[idx]  += as3*H3st_qqp_1;
	    qqp_2[idx]  += as3*H3st_qqp_2;
	    qqbp_1[idx] += as3*H3st_qqbp_1;
	    qqbp_2[idx] += as3*H3st_qqbp_2;
	    gg[idx]     += as3*H3st_gg;

	    if (opts.order_sudak == 3)
	      continue;

	    //NNNNLO
	    complex <double> H3st_mu0_qqb    = C3qq_1[ii1] + C3qq_2[ii2] + C1qq_1[ii1]*C2qq_2[ii2] + C1qq_2[ii2]*C2qq_1[ii1];
	    complex <double> H3st_mu0_qg_1   = C3qg_1[ii1] + C1qq_2[ii2]*C2qg_1[ii1] + C2qq_2[ii2]*C1qg_1[ii1] + H2q*C1qg_1[ii1];
	    complex <double> H3st_mu0_qg_2   = C3qg_2[ii2] + C1qq_1[ii1]*C2qg_2[ii2] + C2qq_1[ii1]*C1qg_2[ii2] + H2q*C1qg_2[ii2];
	    complex <double> H3st_mu0_qq_1   = C3qqb_1[ii1] + C1qq_2[ii2]*C2qqb_1[ii1];
	    complex <double> H3st_mu0_qq_2   = C3qqb_2[ii2] + C1qq_1[ii1]*C2qqb_2[ii2];
	    complex <double> H3st_mu0_qqp_1  = C3qqbp_1[ii1]+ C1qq_2[ii2]*C2qqbp_1[ii1];
	    complex <double> H3st_mu0_qqp_2  = C3qqbp_2[ii2]+ C1qq_1[ii1]*C2qqbp_2[ii2];
	    complex <double> H3st_mu0_qqbp_1 = C3qqp_1[ii1] + C1qq_2[ii2]*C2qqp_1[ii1];
	    complex <double> H3st_mu0_qqbp_2 = C3qqp_2[ii2] + C1qq_1[ii1]*C2qqp_2[ii2];
	    complex <double> H3st_mu0_gg     = C1qg_1[ii1]*C2qg_2[ii2] + C1qg_2[ii2]*C2qg_1[ii1];
	    complex <double> H3st_mu0_qbg_1  = C2qqb_2[ii2]*C1qg_1[ii1];
	    complex <double> H3st_mu0_qbg_2  = C2qqb_1[ii1]*C1qg_2[ii2];
	    complex <double> H3st_mu0_qpg_1  = C2qqp_2[ii2]*C1qg_1[ii1];
	    complex <double> H3st_mu0_qpg_2  = C2qqp_1[ii1]*C1qg_2[ii2];
	    complex <double> H3st_mu0_qbpg_1 = C2qqbp_2[ii2]*C1qg_1[ii1];
	    complex <double> H3st_mu0_qbpg_2 = C2qqbp_1[ii1]*C1qg_2[ii2];

	    complex <double> H4st_qqb    = h4q_3 * H0st_mu0_qqb + h4q_2 * H1st_mu0_qqb  + h4q_1 * H2st_mu0_qqb    + h4q_0 * H3st_mu0_qqb   ;
	    complex <double> H4st_qg_1   =                        h4q_2 * H1st_mu0_qg_1 + h4q_1 * H2st_mu0_qg_1   + h4q_0 * H3st_mu0_qg_1  ;
	    complex <double> H4st_qg_2   =                        h4q_2 * H1st_mu0_qg_2 + h4q_2 * H2st_mu0_qg_2   + h4q_0 * H3st_mu0_qg_2  ;
	    complex <double> H4st_qq_1   =                                                h4q_1 * H2st_mu0_qq_1   + h4q_0 * H3st_mu0_qq_1  ;
	    complex <double> H4st_qq_2   =                                                h4q_2 * H2st_mu0_qq_2   + h4q_0 * H3st_mu0_qq_2  ;
	    complex <double> H4st_qqp_1  =                                                h4q_1 * H2st_mu0_qqp_1  + h4q_0 * H3st_mu0_qqp_1 ;
	    complex <double> H4st_qqp_2  =                                                h4q_2 * H2st_mu0_qqp_2  + h4q_0 * H3st_mu0_qqp_2 ;
	    complex <double> H4st_qqbp_1 =                                                h4q_1 * H2st_mu0_qqbp_1 + h4q_0 * H3st_mu0_qqbp_1;
	    complex <double> H4st_qqbp_2 =                                                h4q_2 * H2st_mu0_qqbp_2 + h4q_0 * H3st_mu0_qqbp_2;
	    complex <double> H4st_gg     =                                                h4q_1 * H2st_mu0_gg     + h4q_0 * H3st_mu0_gg    ;
	    complex <double> H4st_qbg_1  =                                                                          h4q_0 * H3st_mu0_qbg_1 ;
	    complex <double> H4st_qbg_2  =                                                                          h4q_0 * H3st_mu0_qbg_2 ;
	    complex <double> H4st_qpg_1  =                                                                          h4q_0 * H3st_mu0_qpg_1 ;
	    complex <double> H4st_qpg_2  =                                                                          h4q_0 * H3st_mu0_qpg_2 ;
	    complex <double> H4st_qbpg_1 =                                                                          h4q_0 * H3st_mu0_qbpg_1;
	    complex <double> H4st_qbpg_2 =                                                                          h4q_0 * H3st_mu0_qbpg_2;

	    //Additional Cgq and Cgg terms
	    //H4st_qqb    += -       beta0*(C2qg_1[ii1]*C1gq_1[ii1]+C2qg_2[ii2]*C1gq_2[ii2])*LQ +       beta0*(C1qg_1[ii1]*C2gq_1[ii1]+C1qg_2[ii2]*C2gq_2[ii2])*LQ;
	    //H4st_qg_1   += - 1./2.*beta0*C2qg_1[ii1]*C1gg_1[ii1]*LQ + 1./2.*beta0*C1qg_1[ii1]*C2gg_1[ii1]*LQ;
	    //H4st_qg_2   += - 1./2.*beta0*C2qg_2[ii2]*C1gg_2[ii2]*LQ + 1./2.*beta0*C1qg_2[ii2]*C2gg_2[ii2]*LQ;
	    //H4st_qq_1   += - 1./2.*beta0*C2qg_1[ii1]*C1gq_1[ii1]*LQ + 1./2.*beta0*C1qg_1[ii1]*C2gq_1[ii1]*LQ;
	    //H4st_qq_2   += - 1./2.*beta0*C2qg_2[ii2]*C1gq_2[ii2]*LQ + 1./2.*beta0*C1qg_2[ii2]*C2gq_2[ii2]*LQ;
	    //H4st_qqp_1  += - 1./2.*beta0*C2qg_1[ii1]*C1gq_1[ii1]*LQ + 1./2.*beta0*C1qg_1[ii1]*C2gq_1[ii1]*LQ;
	    //H4st_qqp_2  += - 1./2.*beta0*C2qg_2[ii2]*C1gq_2[ii2]*LQ + 1./2.*beta0*C1qg_2[ii2]*C2gq_2[ii2]*LQ;
	    //H4st_qqbp_1 += - 1./2.*beta0*C2qg_1[ii1]*C1gq_1[ii1]*LQ + 1./2.*beta0*C1qg_1[ii1]*C2gq_1[ii1]*LQ;
	    //H4st_qqbp_2 += - 1./2.*beta0*C2qg_2[ii2]*C1gq_2[ii2]*LQ + 1./2.*beta0*C1qg_2[ii2]*C2gq_2[ii2]*LQ;
	    
	    //Mixed resummation-renormalization scale variations
	    if (opts.order_hcoef >= 4)
	      {
		H4st_qqb    += -3*beta0*H3st_qqb*LR    -(2*beta1*LR + 3*pow(beta0,2)*pow(LR,2))*H2st_qqb - (beta2*LR + 5./2.*beta1*beta0*pow(LR,2) + pow(beta0,3)*pow(LR,3))*H1st_qqb;
		H4st_qg_1   += -3*beta0*H3st_qg_1*LR   -(2*beta1*LR + 3*pow(beta0,2)*pow(LR,2))*H2st_qg_1;
		H4st_qg_2   += -3*beta0*H3st_qg_2*LR   -(2*beta1*LR + 3*pow(beta0,2)*pow(LR,2))*H2st_qg_2;
		H4st_qq_1   += -3*beta0*H3st_qq_1*LR;
		H4st_qq_2   += -3*beta0*H3st_qq_2*LR;
		H4st_qqp_1  += -3*beta0*H3st_qqp_1*LR;
		H4st_qqp_2  += -3*beta0*H3st_qqp_2*LR;
		H4st_qqbp_1 += -3*beta0*H3st_qqbp_1*LR;
		H4st_qqbp_2 += -3*beta0*H3st_qqbp_2*LR;
		H4st_gg     += -3*beta0*H3st_gg*LR;
	      }
	
	    qqb[idx]    += as4*H4st_qqb;
	    qg_1[idx]   += as4*H4st_qg_1;
	    qg_2[idx]   += as4*H4st_qg_2;
	    qq_1[idx]   += as4*H4st_qq_1;
	    qq_2[idx]   += as4*H4st_qq_2;
	    qqp_1[idx]  += as4*H4st_qqp_1;
	    qqp_2[idx]  += as4*H4st_qqp_2;
	    qqbp_1[idx] += as4*H4st_qqbp_1;
	    qqbp_2[idx] += as4*H4st_qqbp_2;
	    gg[idx]     += as4*H4st_gg;
	    qbg_1[idx]  += as4*H4st_qbg_1;
	    qbg_2[idx]  += as4*H4st_qbg_2;
	    qpg_1[idx]  += as4*H4st_qpg_1;
	    qpg_2[idx]  += as4*H4st_qpg_2;
	    qbpg_1[idx] += as4*H4st_qbpg_1;
	    qbpg_2[idx] += as4*H4st_qbpg_2;
	  }
}
